from pluggy import HookimplMarker
from pluggy import HookspecMarker

hookspec = HookspecMarker("dsbuild")
hookimpl = HookimplMarker("dsbuild")


@hookspec
def register_formats(register):
    "Register classes to handle different file formats"



import click
from datasette_build.core import get_formats
import itertools
import pathlib
import sqlite_utils
from sqlite_utils.utils import TypeTracker


@click.command()
@click.argument(
    "db_path",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
)
@click.argument(
    "directory_path",
    type=click.Path(file_okay=False, dir_okay=True, allow_dash=False),
)
def cli(db_path, directory_path):
    """Build a directory full of files into a SQLite database"""
    formats = {}
    for format, ext in get_formats():
        formats[ext] = format

    db = sqlite_utils.Database(db_path)

    # Iterate through every file and directory in directory_path\
    path = pathlib.Path(directory_path)
    for child in path.iterdir():
        if child.is_file() and not child.stem.startswith("."):
            # It's a file, act based on its extension
            extension = child.suffix.lstrip(".")
            if extension in formats:
                rows = formats[extension].parse_file(child.open())
                first_row = next(rows)
                columns = list(first_row.keys())
                rows = itertools.chain([first_row], rows)
                pk = None
                if "id" in columns:
                    pk = "id"
                table_name = child.stem
                detect_types = getattr(formats[extension], "detect_types", False)
                tracker = None
                if detect_types:
                    tracker = TypeTracker()
                    rows = tracker.wrap(rows)
                db[table_name].insert_all(rows, pk=pk, replace=True)
                if tracker is not None:
                    db[table_name].transform(types=tracker.types)
                row_count = db[table_name].count
                click.echo(
                    "{} row{} in {}".format(
                        row_count, "" if row_count == 1 else "s", table_name
                    ),
                    err=True,
                )
        elif child.is_dir():
            click.echo("    is_dir: " + str(child), err=True)


if __name__ == "__main__":
    cli()



from datasette_build import hookimpl
import csv
import json


@hookimpl
def register_formats(register):
    register(CsvFormat(), extension="csv")
    register(CsvFormat(dialect=csv.excel_tab), extension="tsv")
    register(JsonFormat(), extension="json")


class CsvFormat:
    detect_types = True

    def __init__(self, dialect=csv.excel):
        self.dialect = dialect

    def parse_file(self, fp):
        yield from csv.DictReader(fp, dialect=self.dialect)


class JsonFormat:
    def parse_file(self, fp):
        yield from json.load(fp)



from datasette_build.cli import cli
from click.testing import CliRunner
import pathlib
import pytest
import sqlite_utils

DEMO_DIR = pathlib.Path(__file__).parent / "demo"


@pytest.fixture(scope="session")
def demo_db(tmp_path_factory):
    tmpdir = tmp_path_factory.mktemp("built")
    db_path = str(tmpdir / "demo.db")
    runner = CliRunner(mix_stderr=False)
    result = runner.invoke(
        cli, [str(tmpdir / "demo.db"), str(DEMO_DIR)], catch_exceptions=False
    )
    assert result.exit_code == 0
    expected_lines = [
        "10 rows in museums",
        "10 rows in cities",
        "10 rows in countries",
    ]
    for line in expected_lines:
        assert line in result.stderr
    return sqlite_utils.Database(db_path)


def test_demo_db_tables(demo_db):
    assert set(demo_db.table_names()) == {"cities", "countries", "museums"}


def test_demo_db_tsv(demo_db):
    assert demo_db["countries"].count == 10
    assert demo_db["countries"].columns_dict == {
        "id": str,
        "name": str,
        "population": int,
    }
    # Spot check
    assert next(demo_db["countries"].rows_where("id = 'US'")) == {
        "id": "US",
        "name": "United States",
        "population": 331002651,
    }
    assert next(demo_db["countries"].rows_where("id = 'EG'")) == {
        "id": "EG",
        "name": "Egypt",
        "population": 102334404,
    }


def test_demo_db_csv(demo_db):
    cities = demo_db["cities"]
    assert cities.count == 10
    assert cities.columns_dict == {
        "id": str,
        "name": str,
        "latitude": float,
        "longitude": float,
        "country": str,
    }
    # Spot check
    assert next(cities.rows_where("id = 'nyc'")) == {
        "id": "nyc",
        "name": "New York City",
        "latitude": 40.7128,
        "longitude": -74.006,
        "country": "US",
    }
    assert next(cities.rows_where("id = 'syd'")) == {
        "id": "syd",
        "name": "Sydney",
        "latitude": -33.8688,
        "longitude": 151.2093,
        "country": "AU",
    }


def test_demo_db_json(demo_db):
    museums = demo_db["museums"]
    assert museums.count == 10
    assert museums.columns_dict == {
        "id": int,
        "name": str,
        "city_id": str,
    }
    # Spot check
    assert next(museums.rows_where("id = 1")) == {
        "id": 1,
        "name": "Metropolitan Museum of Art",
        "city_id": "nyc",
    }
    assert next(museums.rows_where("id = 10")) == {
        "id": 10,
        "name": "Chhatrapati Shivaji Maharaj Vastu Sangrahalaya",
        "city_id": "mum",
    }



import pluggy
import importlib
from . import hookspecs

pm = pluggy.PluginManager("dsbuild")
pm.add_hookspecs(hookspecs)
pm.load_setuptools_entrypoints("dsbuild")

mod = importlib.import_module("datasette_build.default_formats")
pm.register(mod)


def get_formats():
    formats = []

    def register(format, extension=None):
        formats.append((format, extension))

    pm.hook.register_formats(register=register)
    return formats
